## 0. AppendFunctions.R
## Fracfocus database - append geochemical/parameter data to each distinct well in FFU
## Andrew Sumner 2-Jan-2020

# install.packages(c("deSolve", "geosphere")) for coordinate distance functions
library(deSolve)
library(geosphere)

## I found that calculating the mode is not native to R, so I found one to write in. Need it for later
getmode <- function(v) {
  uniqv <- unique(v)
  uniqv[which.max(tabulate(match(v, uniqv)))]
}

## Add in the geologic basin, according to USGS database. Useful for regional classification
Add_basin <- function(wells,usgs) {
  colnames(wells) <- tolower(colnames(wells)) #normalize caps in column names to lower
  colnames(usgs) <- tolower(colnames(usgs)) 
  basin <- rep(NA,nrow(wells)) # initialize an empty column for basin data
  for (i in 1:nrow(wells)) {
    well <- wells[i,]  # going one well at a time...
    u <- well[,c('longitude','latitude')] # coordinates for well[i]
    v <- usgs[,c('longitude','latitude')] # coordinates for all usgs data
    usgs$dist <- t(distm(u,v))  # calculate the distance of your well from usgs data
    closest <- head(usgs[order(usgs$dist),],10)  # determine the closest 10 wells (can change)
    basin[i] <- getmode(closest$basin)  # determine your wells basin by most common of 10 closest
  }
  result <- data.frame(matrix(c(basin),ncol=1))
  colnames(result) <- c('basin')
  return(result)
}

# I found it convenient to do basin and Cl at the same time since they use the same method
Add_basin_Cl <- function(wells,usgs) {
  colnames(wells) <- tolower(colnames(wells)) # normalize caps in column names to lower
  colnames(usgs) <- tolower(colnames(usgs))
  basin <- rep(NA,nrow(wells)) # initialize an empty column for basin data
  Cl_m <- rep(0,nrow(wells))   # same for mean Cl data
  Cl_sd <- rep(0,nrow(wells))  # and standard deviation
  for (i in 1:nrow(wells)) {
    well <- wells[i,]   # going one well at a time...
    u <- well[,c('longitude','latitude')]  # coordinates for well[i]
    v <- usgs[,c('longitude','latitude')]  # coordinates for USGS wells
    usgs$dist <- t(distm(u,v))   # calculate distance between well[i] and USGS wells
    closest <- head(usgs[order(usgs$dist),],10)  # determine closest n=10 wells
    basin[i] <- getmode(closest$basin)     # set basin as most common among closest 10
    Cl_m[i] <- mean(closest$cl)    # take average of Cl values
    Cl_sd[i] <- sd(closest$cl)     # report standard deviation of the sample
  }
  result <- data.frame(matrix(c(basin,Cl_m,Cl_sd),ncol=3))
  colnames(result) <- c('basin','Cl_m','Cl_sd')
  return(result)
}

Add_pH <- function(wells,usgs) {
  colnames(wells) <- tolower(colnames(wells)) # normalize caps in column names to lower
  colnames(usgs) <- tolower(colnames(usgs))
  pH_m <- rep(NA,nrow(wells))  # initialize an empty column for basin data
  pH_sd <- rep(NA,nrow(wells)) # and standard deviation
  for (i in 1:nrow(wells)) {
    well <- wells[i,]  # going one well at a time
    u <- well[,c('longitude','latitude')]  # coordinates for well[i]
    v <- usgs[,c('longitude','latitude')]  # coordinates for USGS data
    usgs$dist <- t(distm(u,v))             # calculate distances
    closest <- head(usgs[order(usgs$dist),],20)  # look now at the closest 20 (bigger range)
    pH_m[i] <- mean(closest[!is.na(closest$ph),]$ph)  # take average of pH values
    pH_sd[i] <- sd(closest[!is.na(closest$ph),]$ph)   # and standard deviation
  }
  result <- data.frame(matrix(c(pH_m,pH_sd),ncol=2))
  colnames(result) <- c('pH_m','pH_sd')
  return(result)
}

## I made this function, and it's the same, but there is so little DOC data that I don't use it
Add_DOC <- function(wells,usgs) {
  colnames(wells) <- tolower(colnames(wells))
  colnames(usgs) <- tolower(colnames(usgs))
  doc_m <- rep(NA,nrow(wells))
  doc_sd <- rep(NA,nrow(wells))
  for (i in 1:nrow(wells)) {
    well <- wells[i,]
    u <- well[,c('longitude','latitude')]
    v <- usgs[,c('longitude','latitude')]
    usgs$dist <- t(distm(u,v))
    closest <- head(usgs[order(usgs$dist),],10)
    doc_m[i] <- mean(closest[!is.na(closest$doc),]$doc)
    doc_sd[i] <- sd(closest[!is.na(closest$doc),]$doc)
  }
  result <- data.frame(matrix(c(doc_m,doc_sd),ncol=2))
  colnames(result) <- c('doc_m','doc_sd')
  return(result)
}

## adding in Br and I, this time by basin averages
Add_Br_I <- function(wells,usgs) {
  colnames(wells) <- tolower(colnames(wells))  # normalize columns
  colnames(usgs) <- tolower(colnames(usgs))    # same
  result <- wells 
  result$Br_m <- rep(NA,nrow(wells))  # set up a dataframe to collect data
  result$Br_sd <- rep(NA,nrow(wells))
  result$I_m <- rep(NA,nrow(wells))
  result$I_sd <- rep(NA,nrow(wells))
  basins <- unique(result$basin)  # get a list of basins
  for (i in 1:length(basins)) {   # iterate by basin, calculate average values, and apply to data
    b <- basins[[i]]
    result[result$basin == b,]$Br_m <- mean(usgs[usgs$basin == b & !is.na(usgs$br) ,]$br)
    result[result$basin == b,]$Br_sd <- sd(usgs[usgs$basin == b & !is.na(usgs$br) ,]$br)
    result[result$basin == b,]$I_m <-  mean(usgs[usgs$basin == b & !is.na(usgs$i) ,]$i)
    result[result$basin == b,]$I_sd <-  sd(usgs[usgs$basin == b & !is.na(usgs$i) ,]$i)
  }
  return(result[,c('Br_m','Br_sd','I_m','I_sd')])
}

# Function to add temp values to data w/ lat/long from NGDS database or subset(NGDS), same as Cl
Add_Temp <- function(wells,ngds) {
  colnames(wells) <- tolower(colnames(wells))  
  colnames(ngds) <- tolower(colnames(ngds))
  # temp <- rep(NA,nrow(wells))
  temp_m <- rep(NA,nrow(wells))
  temp_sd <- rep(NA,nrow(wells))
  for (i in 1:nrow(wells)) {
    well <- wells[i,]
    u <- well[,c('longitude','latitude')]
    v <- ngds[,c('longitude','latitude')]
    ngds$dist <- t(distm(u,v))
    closest <- head(ngds[order(ngds$dist),],10)
    temp_m[i] <- mean(closest$bht)
    temp_sd[i] <- sd(closest$bht)
  }
  result <- data.frame(matrix(c(temp_m,temp_sd),ncol=2))
  colnames(result) <- c('Temp_m','Temp_sd')
  return(result)
}
